//Components
import Utils  from "./API/IKUtils";
import IKDB from "./Database/IKDB";
import IKLogger from "./Logger/IKLogger";
import IKGlobals from './IKGlobals';
import IKValidation from './Validation/IKValidation';
import IKRequest from './API/IKRequest';
import IKResponse, { IKBadRequestResponse } from './API/IKResponse';
//
const logger = new IKLogger({/*COFIG S3/SQS HERE*/}, Utils.logLevel());;
//
export default class IKTransaction {
  constructor(event, context) {
    //
    this._event = event;
    this._context = context;
    //
    this.db = new IKDB();
    this.request = new IKRequest(this._event, this._context);
    this.validator = new IKValidation();
  }

  //Main interface
  async execute(executionFunc) {
    await this._executeLoggerFlush(async ()=>{
      await this._executeDBTransactions(async ()=>{
        return await this._execute(executionFunc);
      });
    });
  }
  //Executions
  async _execute(executionFunc) {
    let executionFailed = true; //failled til we say no!
    //safe execution handler
    try { //Execute
      console.debug("Starting main request code");
      let response = await executionFunc(this);
      //Answer client
      if (response && response instanceof IKResponse) response.build(this._context, this.request.getRequestID());
      else {
        IKBadRequestResponse(IKGlobals.ErrorResponseInvalidServerResponse, IKGlobals.ErrorCode_APIError).build(this._context, this.request.getRequestID());
        console.error("Invalid response object from main request code.");
      }
      executionFailed = false;
    } catch (e) { /*EXECUTION FAIL*/
      console.error('Exception when executing main request code.');
      console.exception(e);
      if (executionFailed) IKBadRequestResponse(IKGlobals.ErrorResponseUnhandledError, IKGlobals.ErrorCode_APIError).build(this._context, this.request.getRequestID());
    } return executionFailed;
  }
  async _executeDBTransactions(safeExecution) {
    try {
      let executionFailed = await safeExecution();
      if (!executionFailed) await this.db.runQueuedQueries();
      else console.log("Skipping pending DB transactions. Main code failed!");
    } catch (e) {
      console.error('Exception when executing DB transactions.');
      console.log(e);
    }
  }
  async _executeLoggerFlush(safeExecution) {
    try {
      await safeExecution();
      await logger.flushLogs();
    } catch (e) {
      console.error('Exception when flushing logs.');
      console.exception(e);
    } finally {
      console.debug("Transaction ended");
    }
  }
}
